<?php

use Teknisa\MAN\Services\ConsolidacaoAutomatica;
use Teknisa\MAN\Services\ConsolidacaoVendasService;

class ExecConsVen {
    protected $cdFilial;
    protected $cdLoja;
    protected $dtIniConsVenAut;
    protected $cdFiliProc;
    protected $cdOperador;
    protected $nrorg;
    protected $paravendParams;
    protected $filterParams;
    protected $lojaParams;
    protected $produtos;
    protected $cdTipoReti;
    protected $codReferencia = '';
    
    protected $consolidationFileLogService;
    
    /**
     * Metodo Construtor.
     * 
     * @param $cdFilial
     * @param $cdLoja
     * @param $dtIniConsVenAut
     * @param $cdFiliProc
     * @param $cdOperador
     * @param $nrorg
     * @param $cdTipoReti
     */
    public function __construct($cdFilial, $cdLoja, $dtIniConsVenAut, $cdFiliProc, $cdOperador, $nrorg, $cdTipoReti) {
        set_time_limit(43200);

        $this->cdFilial = $cdFilial;
        $this->cdLoja = $cdLoja;
        $this->dtIniConsVenAut = $dtIniConsVenAut;
        $this->cdFiliProc = $cdFiliProc;
        $this->cdOperador = $cdOperador;
        $this->nrorg = $nrorg;
        $this->cdTipoReti = $cdTipoReti;
        
        $this->consolidationFileLogService = ConsolidacaoVendasService::getLogService();

        $this->paravendParams = ConsolidacaoVendasService::getParavendParams($cdFilial, $nrorg);
    }

    /**
     * Metodo que executa a consolidação automática das vendas.
     */
    public function executa() {
        try {
            // pega os produtos
            $this->produtos = $this->findProducts();
            
            // verifica a consolidacao
            $verificaConsolidacao = $this->verificaConsolidacao();

            if($verificaConsolidacao['status']) {
                // agrupa a consolidacao
                $groupConsolidacao = $this->groupConsolidacao();
                
                if($groupConsolidacao['status']) {
                    // agrupa os produtos
                    $groupProdutos = $this->groupProdutos($groupConsolidacao);
                    
                    if($groupProdutos['status']) {
                        $insertConsolidation = $this->insertConsolidation($groupProdutos['message'], $groupProdutos['groupConsolidation']['data']);
                        
                        
                    } else {
                        echo $groupProdutos['message'];
                    }
                } else {
                    echo $groupConsolidacao['message'];
                }
            } else {
                echo $verificaConsolidacao['message'];
            }
        } catch(\Exception $e) {
			echo $e->getMessage();
        }
    }

    /**
     * Metodo para buscar os produtos ainda não consolidados no dia.
     */
    private function findProducts() {
        $params = [
            'CDFILIAL'  => $this->cdFilial,
            'CDLOJA'    => $this->cdLoja,
            'NRORG'     => $this->nrorg,
            'DTINICIAL' => date('d/m/Y'),
            'DTFINAL'   => date('d/m/Y')
        ];
        
        $produtos = ConsolidacaoAutomatica::buscaProdutos($params);
        
        return $produtos;
    }
    
    /**
     * Metodo que valida se a Consolidação pode ou não ser realizada pela parametrização.
     */
    private function verificaConsolidacao() {
        $stockRestriction = $this->stockRestriction();

        if($stockRestriction['status']) {
            return $this->dateRestriction($stockRestriction['message']);
        } else {
            //return $this->returnPromise($stockRestriction); // ?
        }
    }

    /**
     * Metodo que verifica se Filial controla estoque.
     * @return array 
     */
    private function stockRestriction() {
        if($this->paravendParams['IDCTRLESTQ'] == "S") {
            $status = true;
            $message = "Verificando parametrização Controle de Estoque...";
        } else {
            $status = false;
            $message = "Filial deve controlar estoque para realizar a consolidação das vendas. Realizar configuração na tela Parâmetros de Estoque!";
        }
        
        return [
            'status'  => $status,
            'message' => $message
        ];
    }

    /**
     * Metodo que verifica se existe restrição de data.
     * @return array
     */
    private function dateRestriction($message) {
        echo $message;
        
        $params = [
            'filterQuery' => [],
            'selectQuery' => []
        ];

        /*
            IDCONSUMIOBR => 1. Não Permite consolidar vendas caso existam datas não consolidadas anteriormente;
                            2. Permite consolidar vendas caso existam datas não consolidadas;
            DTINITESCONS => Data limite para conslidacao (Parâmetros da Unidade - Consolidação de Vendas - Restrição de Datas);
            IDUTILALMOX  => Utiliza Controle de Local de Estoque (S/N)
            IDUTILALMOX  => Utiliza Controle por Almoxarifado (S/N)
        */

        if($this->paravendParams['IDCONSUMIOBR'] == "1") {
            try {
                $params['filterQuery'] = [
                    'CDFILIAL'     => $this->cdFilial,
                    'DTINITESCONS' => ($this->paravendParams['DTINITESCONS'] != "") ? $this->paravendParams['DTINITESCONS'] : "01/01/1899",
                    'DTINICIAL'    => date("d/m/Y"),
                    'DTFINAL'      => date("d/m/Y"),
                    'NRORG'        => $this->nrorg
                ];
    
                $params['selectQuery'] = [
                    'IDUTILALMOX'  => $this->paravendParams['IDUTILALMOX'],
                    'IDUTILLCESTQ' => $this->paravendParams['IDUTILLCESTQ'],
                    'CDLOJA'       => $this->cdLoja
                ];
    
                $dateRestriction = ConsolidacaoVendasService::dateRestriction($params['filterQuery'], $params['selectQuery']);
    
                if($dateRestriction['status']) {
                    $status = true;
                    $message = "Agrupando itens para consolidação...";
                    $this->lojaParams = $dateRestriction['lojaResults'];
                    
                    return [
                        'status'  => $status,
                        'message' => $message
                    ];
                } else {
                    $status = true;
                    $message = "Operação bloqueada. Existem vendas com data anterior que ainda não foram consolidadas. ". $dateRestriction['message'];
                    
                    return [
                        'status'  => $status,
                        'message' => $message
                    ];
                }
            } catch(\Exception $e) {
                $status = true;
                $message = "Conexão perdida com o servidor. Consolidação de Vendas cancelada! (Erro ao verificar existência de restrição de datas)";
                
                return [
                    'status'  => $status,
                    'message' => $message
                ];
            }
        } else {
            $this->lojaParams = null;
            $status = true;
            $message = "Agrupando itens para consolidação...";
            
            return [
                'status'  => $status,
                'message' => $message
            ];
        }
    }

    /**
     * Metodo para agrupar a consolidação.
     * @return array
     */
    private function groupConsolidacao() {
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        $produtos = $this->produtos;
        $filialParams = $this->paravendParams;
        $lojaParams = $this->lojaParams;
        $paramEstoque = [];
        $groupConsolidacao = [];
        
        $filterParams = [
            'CDFILIAL'    => $this->cdFilial,
            'CDLOJA'      => $this->cdLoja,
            'NRORG'       => $this->nrorg,
            'CDTIPORETI'  => $this->cdTipoReti,
            'NRSEQVENDA'  => null,
            'TODASVND'    => 'S',
            'DTINICIAL'   => date('d/m/Y'),
            'DTFINAL'     => date('d/m/Y'),
            'CDCAIXA'     => null,
            'TCDCAIXA'    => 'T',
            'CDCLIENTE'   => null,
            'TCDCLIENTE'  => 'T',
            'CDCCUSCLIE'  => null,
            'TCDCCUSCLIE' => 'T',
            'CDPRODUTO'   => null,
            'TCDPRODUTO'  => 'T',
            'CDTIPOCONS'  => null,
            'TCDTIPOCONS' => 'T',
            'CDOPERADOR'  => $this->cdOperador,
            'CDSETOR'     => null,
            'sendEmail'   => false
        ];
        
        $this->filterParams = $filterParams;
        
        $this->codReferencia = $this->nrorg . $this->cdFilial . $this->cdLoja;
        
        try {
            // excluir arquivo Consolidacao
            if(file_exists('/tmp/RotinaConsolidacaoRetail_'.date_create()->format('Ymd').'_'.$this->codReferencia.'_processes.txt')) {
                unlink('/tmp/RotinaConsolidacaoRetail_'.date_create()->format('Ymd').'_'.$this->codReferencia.'_processes.txt');
            }
            
            // iniciando a consolidacao
            $this->consolidationFileLogService->saveProcessLog('okInitgroupConsolidation', '', $this->codReferencia);
            $paramEstoque = ConsolidacaoVendasService::getParamEstoque($this->nrorg);
            $this->consolidationFileLogService->saveProcessLog('okStockParams', '', $this->codReferencia);
            
            // testa consolidação por data da venda
            $consolidationForSaleDate = ConsolidacaoVendasService::consolidationForSaleDate($produtos, $filterParams, $filialParams, $paramEstoque);
            $this->consolidationFileLogService->saveProcessLog('okConsolidationPerDateTest', '', $this->codReferencia);
            
            if($consolidationForSaleDate['status']) {
                // reúne produtos da consolidação com as observações
                $this->consolidationFileLogService->saveProcessLog('okProductsDoHaveObs', '', $this->codReferencia);
                $groupConsolidation = ConsolidacaoVendasService::groupConsolidation($filterParams, $filialParams, $paramEstoque, $lojaParams, $products);
                $this->consolidationFileLogService->saveProcessLog('okGroupedProductsWithObs', '', $this->codReferencia);
                
                return [
                    'status'             => true,
                    'groupConsolidation' => $groupConsolidation
                ];
            } else {
                $this->consolidationFileLogService->saveProcessLog('okProductsDoNotHaveObs', '', $this->codReferencia);
                
                return [
                    'status'             => true,
                    'groupConsolidation' => $consolidationForSaleDate
                ];
            }
        } catch(\Exception $e) {
            $this->consolidationFileLogService->saveExceptionLog('exceptionIntro', $e, $this->codReferencia);
            
            return [
                'status'             => false,
                'groupConsolidation' => $e
            ];
        }
    }
    
    /**
     * Metodo para agrupar os produtos.
     * @param $data
     * @return array
     */
    private function groupProdutos($data) {
        ini_set('max_execution_time', 4000);
		ini_set('memory_limit', '4096M');
		
        // $message = "Agrupando produtos...";
        
        // agrupa produtos da consolidação
        $this->consolidationFileLogService->saveProcessLog('okGroupProducts', '', $this->codReferencia);
        $groupConsolidation = ConsolidacaoVendasService::agruparProdutosConsolidacao($data['groupConsolidation'][1], $data['groupConsolidation'][2], $data['groupConsolidation'][3], $data['groupConsolidation'][4]);
        $this->consolidationFileLogService->saveProcessLog('okProductsGroupedWithSuccess', '', $this->codReferencia);
        
        $consolidation = $groupConsolidation['consolidation'];
        
        try {
            if(isset($consolidation["data"])) {
                $status = true;
                $message = "Salvando Consolidação de Vendas...";
                
                return [
                    'status'             => $status,
                    'message'            => $message,
                    'groupConsolidation' => $groupConsolidation
                ];
            } else {
                $status = false;
                $message = "Salvando Consolidação de Vendas...";
                
                return [
                    'status'             => $status,
                    'message'            => $message,
                    'groupConsolidation' => $groupConsolidation
                ];
            }
        } catch(\Exception $e) {
            $status  = false;
            $message = "Conexão perdida com o servidor. Consolidação de Vendas cancelada! (Erro ao realizar o agrupamento de produtos)";
            
            return [
                'status'             => $status,
                'message'            => $message,
                'groupConsolidation' => null
            ];
        }
    }
    
    /**
     * Metodo insertConsolidation() - para realizar a inserção da consolidação.
     * 
     * @param $messageLoader
     * @param $dataConsolidation
     * @return array
     */
    private function insertConsolidation($messageLoader, $dataConsolidation) {
        ini_set('max_execution_time', 1800);
		ini_set('memory_limit', '1G');
		
        echo $messageLoader;
        
        $data = $dataConsolidation['consolidation']['data'];
        $dataEstRel = $dataConsolidation['consolidationEstRel']['data'];
        
        $gridItens = $this->produtos;
        
        if(empty($this->filterParams['CDSETOR'])) {
            $this->filterParams['CDSETOR'] = null;
        }
        
        $insertConsolidation = [];
        
        try {
            $insertConsolidation = ConsolidacaoVendasService::insertConsolidation($data, $gridItens, $this->filterParams, $this->filialParams, $dataEstRel);
            
            if($insertConsolidation['status'] == "FAIL" && $insertConsolidation['itensInativos']) {
                // $this->popUpIrregEstoq(); ///////
                $status = $result['insertConsolidation']['commited'];
                $message = $result['insertConsolidation']['message'];
                
                return [
                    'status'  => $status,
                    'message' => $message,
                    'data'    => null
                ];
            } else {
                $status = $insertConsolidation['commited'];
                $message = $this->messageConsolidation($insertConsolidation['status'], $insertConsolidation['dateBlock']); 
                
                return [
                    'status'  => $status,
                    'message' => $message,
                    'data'    => $insertConsolidation['irregData'],
                    'block'   => $insertConsolidation['dateBlock']
                ];
            }
        } catch(\Exception $e) {
            $status = false;
            $message = "Conexão perdida com o servidor. Consolidação de Vendas cancelada! (Erro ao inserir e finalizar a Consolição de Vendas)"; 
                
            return [
                'status'  => $status,
                'message' => $message,
                'data'    => null
            ];
        }
    }
    
    private function messageConsolidation($status, $date) {
        $message = "";
        $messageBlock = "";
        
        if($date != "" && !isset($date)) {
            $messageBlock = " Apresentou primeira ocorrência de produtos irregulares na data " . $date . "!";
        }
        
        switch($status) {
            // status 'SUCCESS' para consolidação concluída 
            case "SUCCESS":
                $message = "Consolidação de Vendas finalizada!";
                break;
                
            // status 'FAIL' para consolidação cancelada
            case "FAIL":
                $message = "Operação cancelada!" . $messageBlock;
                break;
            
            // status 'FAIL_DATE' para consolidação cancelada com todas as vendas da data irregulares 
            case "FAIL_DATE":
                $message = "Operação cancelada! Todas as vendas da data " . $date . " apresentaram um ou mais produtos que ficarão irregulares no estoque. Neste caso mesmo com a parametrização adequada não é possível consolidar quando há irregularidade em todas as vendas!";
                break;
        }
        
        return $message;
    }
}

$execConsVen = new ExecConsVen($argv[1], $argv[2], $argv[3], $argv[4], $argv[5], $argv[6], $argv[7]);
$execConsVen->executa();